---
title: "Data wrangling"
output:
  html_document: default
  pdf_document: default
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

```{r, message=FALSE}
library(tidyverse) # dplyr, tidyr, ggplot2 and friends
```


# Data wrangling

Resources:

* <https://dplyr.tidyverse.org/>
* <http://r4ds.had.co.nz/transform.html>


## Data

Focuses on when data is given. Can be entered manually:

```{r}
d1 <- tibble(w = mtcars$wt[1:2], h = mtcars$hp[1:2])
d1
```

`tibble`'s (<https://cran.r-project.org/web/packages/tibble/vignettes/tibble.html>) are modern data frames that work well with `tidyverse`.

"Vanilla"/base R:
```{r}
data.frame(w = mtcars$wt, h = mtcars$hp) # base R
tibble(w = mtcars$wt, h = mtcars$hp) # tidyverse
```



```{r}
d2 <- tribble(
  ~x, ~y,
   1,  2,
   3,  4
)
d2
```

Glue:

```{r}
bind_cols(d1, d2)
bind_rows(d1, d1)
```

## Dataframes vs tibbles

```{r, eval = FALSE}
mtcars
as_tibble(mtcars)
```


Data types:

* `int` stands for integers.
* `dbl` stands for doubles, or real numbers.
* `chr` stands for character vectors, or strings.
* `fctr` stands for factors (categorical variables).
* `dttm` stands for date-times (a date + a time).
* Others: `lgl`, `date`




## The pipe operator, `%>%`

```{r}
filter(mpg, hwy >= 20, hwy <= 26)
```

```{r}
select(filter(mpg, hwy >= 20, hwy <= 26), manufacturer, hwy)

# filter(mpg, hwy >= 20, hwy <= 26)
mpg %>% filter(hwy >= 20, hwy <= 26)
```

* Each data handling operation performed results in one more level of nested calls. 
* The code will quickly become difficult to read
    * Read it from the inner most call and out 
    * Operation description (function name) separated from arguments

The pipe operator. 

A crucial design decision of the `tidyverse`:

* First argument is a `tibble` (or `data.frame`)
* Subsequent arguments say what to do with the `tibble` (or `data.frame`)
* Always return a `tibble` (or `data.frame`)

This enables the so-called piping operator, `%>%`. Example:

# f(x, ...) === x %>% f(...)

```{r}
sum(sqrt(1:10))
1:10 %>% sqrt() %>% sum()
```

```{r}
filter(mpg, hwy >= 20, hwy <= 26) %>% 
  select(manufacturer, hwy)

mpg %>% 
  filter(hwy >= 20, hwy <= 26) %>% 
  select(manufacturer, hwy)
```

## Operations

* `filter()`: keep rows matching criteria
* `select()`: pick/reorder columns by name
* `rename()`: rename columns by name
* `arrange()`: reorder rows
* `mutate()`: add new or modify existing variables
* `summarise()`: perform aggregation operations
* `group_by()`: introduce aggregation groupings
* `count()`: a short-hand for `group_by()` and `summarise()`

A ton of helper functions.

## Examples

```{r}
mpg %>% select(manufacturer, hwy)
```

```{r}
mpg %>% select(class, everything())
```

```{r}
mpg %>% select(-manufacturer, -hwy)
```

```{r}
count(mpg, class)
mpg %>% count(class)
mpg %>% group_by(class) %>% summarise(n = n())

mpg %>% 
  group_by(class) %>% 
  summarise(
    n = n(),
    displ = mean(displ)) %>% 
  arrange(desc(n))
```


```{r}
mpg %>% 
  group_by(manufacturer) %>% 
  summarise(mean_hwy = mean(hwy)) %>% 
  arrange(desc(mean_hwy))

mpg %>% 
  group_by(cyl, trans) %>% 
  summarise(mean_hwy = mean(hwy)) %>% 
  arrange(cyl, desc(mean_hwy))
```

```{r}
mpg %>% 
  group_by(cyl, trans) %>% 
  summarise(mean_hwy = mean(hwy)) %>% 
  arrange(cyl, desc(mean_hwy)) %>% 
  identity()

mpg %>% 
  group_by(cyl, trans) %>% 
  summarise(mean_hwy = mean(hwy)) %>% 
  arrange(cyl, desc(mean_hwy)) %>% 
  I()

mpg %>% 
  group_by(cyl, trans) %>% 
  summarise(mean_hwy = mean(hwy)) %>% 
  arrange(cyl, desc(mean_hwy)) %>% 
  I
```


```{r}
mpg %>% 
  group_by(class, displ) %>% 
  summarise(mean_hwy = mean(hwy))
```

```{r}
ggplot(mpg, aes(displ, color = class)) + 
  geom_density()

d_tmp <- mpg %>% 
  group_by(class) %>% 
  summarise(displ = mean(displ))
d_tmp

ggplot(mpg, aes(displ, color = class)) + 
  geom_density() +
  geom_vline(data = d_tmp, aes(xintercept = displ, color = class))

ggplot(mpg, aes(displ, color = class)) + 
  geom_density() +
  geom_vline(data = d_tmp, aes(xintercept = displ, color = class)) +
  facet_wrap(~ class)
```

## Long/wide

### Wide to long

```{r}
mpg

mpg %>% 
  select(class, hwy, cty)

mpg %>% 
  select(class, hwy, cty) %>% 
  gather(type, value, -class) # gather(type, value, hwy, cty)

mpg %>% 
  select(class, hwy, cty) %>% 
  gather(type, value, -class) %>% 
  ggplot(aes(type, value, color = class)) + 
    geom_boxplot()
```


### Long to wide

```{r}
mpg

mpg %>% 
  mutate(drv = paste0("drv_", drv)) %>% 
  select(class, drv, hwy)

d_tmp <- mpg %>% 
  mutate(drv = paste0("drv_", drv)) %>% 
  select(class, drv, hwy) %>% 
  group_by(drv, class) %>% 
  summarise(hwy = mean(hwy))
d_tmp

d_tmp %>% 
  spread(drv, hwy)

d_tmp %>% 
  spread(drv, hwy, fill = 0)
```



## List columns

```{r}
ggplot(mpg, aes(displ, hwy)) +
  geom_point()

ggplot(mpg, aes(displ, hwy)) +
  geom_point() + 
  NULL

ggplot(mpg, aes(displ, hwy)) +
  geom_point() +
  facet_wrap(~ class)

summary(lm(hwy ~ displ, mpg))
names(summary(lm(hwy ~ displ, mpg)))

fits <- mpg %>% 
  group_by(class) %>% 
  do(tibble(fit = list(lm(hwy ~ displ, .))))

fits

fits %>% 
  mutate(adjR2 = summary(fit[[1]])$adj.r.squared)

fits %>% 
  rowwise() %>% 
  mutate(adjR2 = summary(fit)$adj.r.squared)
```



## Output

```{r}
mpg %>% 
  group_by(class) %>% 
  summarise(mean_hwy = mean(hwy))
```

```{r}
mpg %>% 
  group_by(class) %>% 
  summarise(mean_hwy = mean(hwy)) %>% 
  pander::pander()
```

```{r}
library(pander)

mpg %>% 
  group_by(class) %>% 
  summarise(mean_hwy = mean(hwy)) %>% 
  pander(digits = 3)
```

